<?php
/*
Copyright 2011  Mobile Sentience LLC  (email : oss@mobilesentience.com)
Copyright 2008  Stephen Carroll  (email : scarroll@virtuosoft.net)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
                                                                 
Plugin Name: Virtual Theme
Plugin URI: http://www.mobilesentience.com/apps/oss/virtual-theme
Description: Virtual Theme allows you to switch themes based on virtual paths, making an entire site accessible through multiple themes based on their URL prefix.

Author: Max Jonathan Spaulding - Mobile Sentience LLC
Version: 1.0.1
Author URI: http://www.mobilesentience.com
*/


$VirtualTheme = new VirtualTheme($_SERVER["REQUEST_URI"]);

function compare($a, $b){
	if (strlen($a['url']) == strlen($b['url'])) {
		return 0;
	}
	return (strlen($a['url']) > strlen($b['url'])) ? -1 : 1;
}

class VirtualTheme {   
    var $blogdescription;
    var $currenturl;
    var $stylesheet;
    var $template;
    var $blogname;
    var $siteurl;
    var $options;
    var $home;
    var $uri;
            
    // Based on the current domain name, load the associated theme data
    function VirtualTheme($requestURI){
        
        // Get default settings
        $this->blogdescription = get_option("blogdescription");
        $this->stylesheet = get_option("stylesheet");
        $this->template = get_option("template");
        $this->blogname = get_option("blogname");
        $this->siteurl = get_option("siteurl"); // WordPress url
        $this->home = get_option("home"); // Blog address url
        $this->currenturl = $requestURI;
        
        // Load domain option settings
        global $wp_version;
        if ($wp_version=="2.6"){
            
            // 2.6 fix, this version unserializes data for some odd reason.
            $this->options = get_option("VirtualTheme_options");
        }else{
            $this->options = unserialize(get_option("VirtualTheme_options"));
        }
        
        if (gettype($this->options)!="array"){
            $this->options = array();
        }
        
        // Locate the matching index for the current domain
		usort($this->options, "compare");
        foreach($this->options as $dt){
            if ($this->urlMatches($dt['url'], $this->currenturl)){
                
                // Update the settings for the matching domain
                $this->blogdescription = $dt['blogdescription'];
                $this->blogname = $dt['blogname'];
                $this->stylesheet = $dt['theme'];
                $this->template = $dt['theme'];

                $this->siteurl = $this->siteurl . $dt['url'];
                $this->home = $this->siteurl;

				$_SERVER["REQUEST_URI"] = $this->getRightMost($this->currenturl, $dt['url']);

            }   
        }
        
        // Apply filters and actions
        add_filter('pre_option_blogdescription', array(&$this, 'getBlogdescription'));
        add_filter('pre_option_stylesheet', array(&$this, 'getStylesheet'));
        add_filter('pre_option_template', array(&$this, 'getTemplate'));
        add_filter('pre_option_blogname', array(&$this, 'getBlogname'));
        add_filter('pre_option_siteurl', array(&$this, 'getSiteurl'));
        add_filter('pre_option_home', array(&$this, 'getHome'));
        add_action('admin_menu', array(&$this, 'displayAdminMenu'));
        
        // Specify uri for admin panels
        $this->uri = '?page=' . $this->getRightMost(__FILE__, 'plugins/');
    }
  
	function urlMatches($prefix, $url){
    	return (strncmp($url, $prefix, strlen($prefix)) == 0);
	}

    // Common string functions
    function getRightMost($sSrc, $sSrch) {        
        for ($i = strlen($sSrc); $i >= 0; $i = $i - 1) {
            $f = strpos($sSrc, $sSrch, $i);
            if ($f !== FALSE) {
               return substr($sSrc,$f + strlen($sSrch), strlen($sSrc));
            }
        }
        return $sSrc;
    }

    function delLeftMost($sSource, $sSearch) {
      for ($i = 0; $i < strlen($sSource); $i = $i + 1) {
        $f = strpos($sSource, $sSearch, $i);
        if ($f !== FALSE) {
           return substr($sSource,$f + strlen($sSearch), strlen($sSource));
           break;
        }
      }
      return $sSource;
    }

    function getLeftMost($sSource, $sSearch) {
      for ($i = 0; $i < strlen($sSource); $i = $i + 1) {
        $f = strpos($sSource, $sSearch, $i);
        if ($f !== FALSE) {
           return substr($sSource,0, $f);
           break;
        }
      }
      return $sSource;
    }
	
	function getThemeTitleByTemplate($template){
        
        // Return descriptive name for a given template name
        $themes = get_themes();
        foreach($themes as $theme){
            if ($template==$theme["Template"]){
                break;
            }
        }
        return $theme["Title"];
    }

    function displayAdminMenu(){
        add_options_page('Virtual Theme Options', 'Virtual Theme', 8, __FILE__, array(&$this, 'createAdminPanel'));
    }
    
    // Return modified data based on the current url
    function getBlogdescription(){
        return $this->blogdescription;
    }
    function getStylesheet(){
        return $this->stylesheet;
    }
    function getTemplate(){
        return $this->template;
    }
    function getBlogname(){
        return $this->blogname;
    }
    function getSiteurl(){
        return $this->siteurl;
    }
    function getHome(){
        return $this->home;
    }
    
    // Create the administration panel
    function createAdminPanel(){
        
        // Check if we need to add a Virtual Path
        if ($_GET['action']=="addVirtualPath"){
            $url['url']=strtolower($_POST['url']);
            $url['theme']=$_POST['theme'];
            $url['blogname']=stripslashes($_POST['blogname']);
            $url['blogdescription']=stripslashes($_POST['blogdescription']);
            array_push($this->options, $url);

            update_option("VirtualTheme_options", serialize($this->options));
        }
        
        // Check if we need to edit a Virtual Path
        if ($_GET['action']=="editVirtualPath"){
            $id = $_GET['id'];
            $this->options[$id]['url']=strtolower($_POST['url']);
            $this->options[$id]['theme']=$_POST['theme'];
            $this->options[$id]['blogname']=stripslashes($_POST['blogname']);
            $this->options[$id]['blogdescription']=stripslashes($_POST['blogdescription']);
            update_option("VirtualTheme_options", serialize($this->options));
        }
        
        // Check if we need to delete one or more Virtual Paths
        if ($_GET['action']=="del" && $_POST['chkDelete']){
            foreach(array_reverse($_POST['chkDelete']) as $id){
                array_splice($this->options,$id,1);
            }
            update_option("VirtualTheme_options", serialize($this->options));
        }
        
        // Check if we should display the edit panel
        if ($_GET['action']=="urlProps"){
            $id = $_GET['id'];
            echo '<div class="wrap">
                    <form name="editVirtualPath" id="editVirtualPath" action="'.$this->uri.'&action=editVirtualPath&id='.$id.'" method="post">
                    <h2>' . __('Edit Virtual Theme') . '</h2>
                    <br class="clear" />
                    <div class="tablenav">
                        <br class="clear" />
                    </div>
                    <br class="clear" />
                    <table class="form-table">
                        <tr class="form-field">
                            <th scope="row" valign="top"><label for="url">Virtual Path</label></th>
                            <td><input name="url" id="url" type="text" value="'.$this->options[$id]['url'].'" size="40" /><br />
                            The Virtual Path that is used to access the site (i.e. www.example.com/VirtualPath).</td>
                        </tr>
                        <tr class="form-field">
                            <th scope="row" valign="top"><label for="theme">Theme</label></th>
                            <td>
                                <select name="theme" id="theme" class="postform" >';
                                $themes = get_themes();
                                foreach($themes as $theme){
                                    if ($theme["Template"]==$this->options[$id]['theme']){
                                        echo '<option value="'.$theme["Template"].'" selected>'.$theme["Name"].'</option>';
                                    }else{
                                        echo '<option value="'.$theme["Template"].'">'.$theme["Name"].'</option>';
                                    }
                                }
            echo '              </select>
                                <br />
                                Specify the theme to use when the site is accessed by the given Virtual Path.
                            </td>
                        </tr>
                        <tr class="form-field">
                            <th scope="row" valign="top"><label for="blogname">Blog Title</label></th>
                            <td><input name="blogname" id="blogname" type="text" value="'.htmlspecialchars ($this->options[$id]['blogname']).'" size="40" /><br />
                            The blog title that will be used when the site is accessed by the given Virtual Path.</td>
                        </tr>
                        <tr class="form-field">
                            <th scope="row" valign="top"><label for="blogname">Tagline</label></th>
                            <td><input name="blogdescription" id="blogdescription" type="text" value="'.htmlspecialchars ($this->options[$id]['blogdescription']).'" size="45" /><br />
                            In a few words, the blogs description when accessed by the given Virtual Path.</td>
                        </tr>
                    </table>
                    <p class="submit"><input type="submit" class="button" name="submit" value="Edit Virtual Path" /></p>
                    </form>
                 </div>';
            return;
        }
        
        // Inject the javascript for delete check all option
        echo '<script language="Javascript">
                (function($){
                    $(function(){
                        $("#chkAll").click(function(){
                            c=this.checked;
                            $(".chkDelete").each(function(i){
                                this.checked=c;
                            })
                        });
                    })
                })(jQuery);
              </script>';
        
        // Create the list
        echo '<div class="wrap">
				<form action="https://www.paypal.com/cgi-bin/webscr" method="post">
				<input name="cmd" type="hidden" value="_s-xclick" />
				<input name="hosted_button_id" type="hidden" value="F6PVX3CMH8MK4" />
                <h2>' . __('Virtual Theme') . ' (<a href="#addVirtualPath">add new</a>)<input alt="PayPal - The safer, easier way to pay online!" name="submit" src="https://www.paypalobjects.com/WEBSCR-640-20110306-1/en_US/i/btn/btn_donate_SM.gif" type="image" /><img src="https://www.paypalobjects.com/WEBSCR-640-20110306-1/en_US/i/scr/pixel.gif" border="0" alt="" width="1" height="1" /></form></h2>
                <form name="urlList" id="urlList" action="'.$this->uri.'&action=del'.'" method="post">
                <br class="clear" />
                <div class="tablenav">
                    <div class="alignleft">
                        <input type="submit" value="Delete" name="deleteit" class="button-secondary delete" />
                    </div>
                    <br class="clear" />
                </div>
                <br class="clear" />
                <table class="widefat">
                <thead>
                    <tr>
                        <th scope="col" class="check-column"><input type="checkbox" id="chkAll" /></th>
                        <th scope="col">Virtual Path</th>
                        <th scope="col">Theme</th>
                        <th scope="col">Blog Title</th>
                        <th scope="col">Tagline</th>
                    </tr>
                </thead>
                <tbody id="the-list" class="list:url">';
        $i=0;
        foreach($this->options as $url){
            echo'   <tr id="url-'.$i.'" ';
            if (!fmod($i,2)){
                echo 'class="alternate">'; 
            }else{
                echo '>';
            }
            echo'       <th scope="row" class="check-column"><input type="checkbox" name="chkDelete[]" class="chkDelete" value="'.$i.'" /></th>
                        <td><a href="'.$this->uri.'&action=urlProps&id='.$i.'"/>'.$url['url'].'</a></td>
                        <td>'.$this->getThemeTitleByTemplate($url['theme']).'</td>
                        <td>'.$url['blogname'].'</td>
                        <td>'.$url['blogdescription'].'</td>
                    </tr>
                ';            
            $i++;
        }
        
        // Create the add form
        echo ' </form>
               <form name="addVirtualPath" id="addVirtualPath" action="'.$this->uri.'&action=addVirtualPath" method="post">';
		echo '   <tr ';
		if (!fmod($i,2)){
			echo 'class="alternate">'; 
		}else{
			echo '>';
		}
		echo '       <th scope="row" class="check-column"><input type="checkbox" name="chkDelete[]" class="chkDelete" disabled="disabled" /></th>
						<td><input name="url" id="url" type="text" value="" size="40" /></td>
						<td>
							<select name="theme" id="theme" class="postform" >';
		$themes = get_themes();
		foreach($themes as $theme){
			echo '       <option value="'.$theme["Template"].'">'.$theme["Name"].'</option>';
		}
		echo '              </select>
                            <br />
							</td>
                            <td><input name="blogname" id="blogname" type="text" value="" size="40" /></td>
                            <td><input name="blogdescription" id="blogdescription" type="text" value="" size="45" /></td>
                    </tr>
					<tr>
						<td colspan="4" align="center"><p class="submit"><input type="submit" class="button" name="submit" value="Add Virtual Path" /></p></td>
					</tr>
                </tbody>
              </table>
            </form>
         </div>';
    }
}       
?>
